@@ -35,7 +35,7 @@ import unipath
 from unipath import pathof
 import unicodedata
 
-_launcher_version=20170227
+_launcher_version=20180723
 
 _PKG_VER = re.compile(r'''<\s*package[^>]*version\s*=\s*["']([^'"]*)['"][^>]*>''',re.IGNORECASE)
 
@@ -49,32 +49,34 @@ _guide_types = ['cover','title-page','toc','index','glossary','acknowledgements'
                 'epigraph','foreward','loi','lot','notes','preface','text']
 
 ext_mime_map = {
-                '.jpg'  : 'image/jpeg',
-                '.jpeg' : 'image/jpeg',
-                '.png'  : 'image/png',
-                '.gif'  : 'image/gif',
-                '.svg'  : 'image/svg+xml',
-                '.xhtml': 'application/xhtml+xml',
-                '.html' : 'application/xhtml+xml',
-                '.htm'  : 'application/xhtml+xml',
-                '.otf'  : 'application/vnd.ms-opentype',
+                '.jpg'   : 'image/jpeg',
+                '.jpeg'  : 'image/jpeg',
+                '.png'   : 'image/png',
+                '.gif'   : 'image/gif',
+                '.svg'   : 'image/svg+xml',
+                '.xhtml' : 'application/xhtml+xml',
+                '.html'  : 'application/xhtml+xml',
+                '.htm'   : 'application/xhtml+xml',
+                '.otf'   : 'application/vnd.ms-opentype',
                 #'.otf'  : 'application/x-font-opentype',
-                '.ttf'  : 'application/x-font-ttf',
-                '.woff' : 'application/font-woff',
-                '.mp3'  : 'audio/mpeg',
-                '.m4a'  : 'audio/mp4',
-                '.mp4'  : 'video/mp4',
-                '.m4v'  : 'video/mp4',
-                '.css'  : 'text/css',
-                '.ncx'  : 'application/x-dtbncx+xml',
-                '.xml'  : 'application/oebs-page-map+xml',
-                '.opf'  : 'application/oebps-package+xml',
-                '.smil' : 'application/smil+xml',
-                '.pls'  : 'application/pls+xml',
-                '.xpgt' : 'application/adobe-page-template+xml',
-                '.js'   : 'text/javascript',
-                #'.js'   : 'application/javascript',
-                '.epub' : 'application/epub+zip',
+                #'.otf'  : 'application/font-sfnt',
+                '.ttf'   : 'application/x-font-ttf',
+                '.woff'  : 'application/font-woff',
+                '.woff2' : 'font/woff2',
+                '.mp3'   : 'audio/mpeg',
+                '.m4a'   : 'audio/mp4',
+                '.mp4'   : 'video/mp4',
+                '.m4v'   : 'video/mp4',
+                '.css'   : 'text/css',
+                '.ncx'   : 'application/x-dtbncx+xml',
+                '.xml'   : 'application/oebs-page-map+xml',
+                '.opf'   : 'application/oebps-package+xml',
+                '.smil'  : 'application/smil+xml',
+                '.pls'   : 'application/pls+xml',
+                '.xpgt'  : 'application/adobe-page-template+xml',
+                #'.js'   : 'text/javascript',
+                '.js'    : 'application/javascript',
+                '.epub'  : 'application/epub+zip',
                 }
 
 mime_base_map = {
@@ -87,6 +89,8 @@ mime_base_map = {
                 'application/x-font-opentype'             : 'Fonts',
                 'application/vnd.ms-opentype'             : 'Fonts',
                 'application/font-woff'                   : 'Fonts',
+                'application/font-sfnt'                   : 'Fonts',
+                'font/woff2'                              : 'Fonts',
                 'audio/mpeg'                              : 'Audio',
                 'audio/mp3'                               : 'Audio',
                 'audio/mp4'                               : 'Audio',
@@ -99,6 +103,7 @@ mime_base_map = {
                 'application/adobe-page-template+xml'     : 'Misc',
                 'application/vnd.adobe-page-template+xml' : 'Misc',
                 'text/javascript'                         : 'Misc',
+                'application/javascript'                  : 'Misc',
                 'application/pls+xml'                     : 'Misc',
                 }
 
@@ -120,6 +125,7 @@ TEXT_MIMETYPES = [
                 'application/adobe-page-template+xml',
                 'application/vnd.adobe-page-template+xml',
                 'text/javascript',
+                'application/javascript'
                 'application/pls+xml'
 ]
 
@@ -145,6 +151,9 @@ class Wrapper(object):
         self.sigil_ui_lang = None
         # Default Sigil spell check dictionary
         self.sigil_spellcheck_lang = None
+        # status of epub inside Sigil (isDirty) and CurrentFilePath of current epub file
+        self.epub_isDirty = False
+        self.epub_filepath = ""
         # File selected in Sigil's Book Browser
         self.selected = []
         cfg = ''
@@ -159,6 +168,8 @@ class Wrapper(object):
                 self.linux_hunspell_dict_dirs = cfg_lst.pop(0).split(":")
             self.sigil_ui_lang = cfg_lst.pop(0)
             self.sigil_spellcheck_lang = cfg_lst.pop(0)
+            self.epub_isDirty = (cfg_lst.pop(0) == "True")
+            self.epub_filepath = cfg_lst.pop(0)
             self.selected = cfg_lst
         os.environ['SigilGumboLibPath'] = self.get_gumbo_path()
 
@@ -199,6 +210,7 @@ class Wrapper(object):
             self.metadataxml = op.get_metadataxml()
         self.other = []  # non-manifest file information
         self.id_to_filepath = {}
+        self.book_href_to_filepath = {}
         self.modified = {}
         self.added = []
         self.deleted = []
@@ -217,7 +229,7 @@ class Wrapper(object):
                 id = self.href_to_id.get(href,None)
             if id is None:
                 self.other.append(book_href)
-                self.id_to_filepath[book_href] = filepath
+                self.book_href_to_filepath[book_href] = filepath
             else:
                 self.id_to_filepath[id] = filepath
 
@@ -710,15 +722,17 @@ class Wrapper(object):
     def readotherfile(self, book_href):
         id = unicode_str(book_href)
         id = unquoteurl(id)
-        if id in self.id_to_href:
+        if id is None:
+            raise WrapperException('None is not a valid book href')
+        if id not in self.other and id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use readfile')
         # handle special case of trying to read the opf after it has been modified
-        if id is not None and id == "OEBPS/content.opf":
+        if id == "OEBPS/content.opf":
             if id in self.modified:
                 return self.build_opf()
-        filepath = self.id_to_filepath.get(id, None)
+        filepath = self.book_href_to_filepath.get(id, None)
         if filepath is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('Book href does not exist')
         basedir = self.ebook_root
         if id in self.added or id in self.modified:
             basedir = self.outdir
@@ -739,11 +753,13 @@ class Wrapper(object):
     def writeotherfile(self, book_href, data):
         id = unicode_str(book_href)
         id = unquoteurl(id)
-        if id in self.id_to_href:
+        if id is None:
+            raise WrapperException('None is not a valid book href')
+        if id not in self.other and id in self.id_to_href:
             raise WrapperException('Incorrect interface routine - use writefile')
-        filepath = self.id_to_filepath.get(id, None)
+        filepath = self.book_href_to_filepath.get(id, None)
         if filepath is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('Book href does not exist')
         if id in PROTECTED_FILES:
             raise WrapperException('Attempt to modify protected file')
         filepath = os.path.join(self.outdir, filepath)
@@ -759,12 +775,14 @@ class Wrapper(object):
     def addotherfile(self, book_href, data) :
         id = unicode_str(book_href)
         id = unquoteurl(id)
+        if id is None:
+            raise WrapperException('None is not a valid book href')
         if id in self.other:
-            raise WrapperException('book href must be unquie')
+            raise WrapperException('Book href must be unique')
         desired_path = id.replace("/",os.sep)
         filepath = os.path.join(self.outdir,desired_path)
         if unipath.isfile(filepath):
-            raise WrapperException('desired path already exists')
+            raise WrapperException('Desired path already exists')
         base = os.path.dirname(filepath)
         if not unipath.exists(base):
             os.makedirs(pathof(base))
@@ -774,16 +792,18 @@ class Wrapper(object):
             fp.write(data)
         self.other.append(id)
         self.added.append(id)
-        self.id_to_filepath[id] = desired_path
+        self.book_href_to_filepath[id] = desired_path
 
     def deleteotherfile(self, book_href):
         id = unicode_str(book_href)
         id = unquoteurl(id)
-        if id in self.id_to_href:
-            raise WrapperException('Incorrect interface routine - use deletefile')
-        filepath = self.id_to_filepath.get(id, None)
         if id is None:
-            raise WrapperException('book href does not exist')
+            raise WrapperException('None is not a valid book hrefbook href')
+        if id not in self.other and id in self.id_to_href:
+            raise WrapperException('Incorrect interface routine - use deletefile')
+        filepath = self.book_href_to_filepath.get(id, None)
+        if filepath is None:
+            raise WrapperException('Book href does not exist')
         if id in PROTECTED_FILES:
             raise WrapperException('attempt to delete protected file')
         add_to_deleted = True
@@ -801,7 +821,7 @@ class Wrapper(object):
                 del self.modified[id]
         if add_to_deleted:
             self.deleted.append(('other', id, book_href))
-        del self.id_to_filepath[id]
+        del self.book_href_to_filepath[id]
 
 
     # utility routine to copy entire ebook to a destination directory
@@ -814,10 +834,18 @@ class Wrapper(object):
         for id in self.id_to_filepath:
             rpath = self.id_to_filepath[id]
             in_manifest = id in self.id_to_mime
-            if in_manifest:
-                data = self.readfile(id)
-            else:
-                data = self.readotherfile(id)
+            data = self.readfile(id)
+            filepath = os.path.join(destdir,rpath)
+            base = os.path.dirname(filepath)
+            if not unipath.exists(base):
+                os.makedirs(base)
+            if isinstance(data,text_type):
+                data = utf8_str(data)
+            with open(pathof(filepath),'wb') as fp:
+                fp.write(data)
+        for id in self.book_href_to_filepath:
+            rpath = self.book_href_to_filepath[id]
+            data = self.readotherfile(id)
             filepath = os.path.join(destdir,rpath)
             base = os.path.dirname(filepath)
             if not unipath.exists(base):
